@@ -4,8 +4,7 @@
 //! of [BlockExtra]
 //!
 //! Blocks could be consumed in 2 different ways:
-//! * Through the [`iter`] or [`par_iter`] method. The latter is to be preferred when performing
-//!   costly task.
+//! * Through the [`iter`] method.
 //! * Via unix pipeline using the [`PipeIterator`]
 //!
 
@@ -24,12 +23,12 @@
 extern crate test;
 
 use bitcoin::BlockHash;
-use log::{error, info, Level};
+use log::{info, Level};
 use std::fs::File;
 
 use std::path::PathBuf;
 use std::sync::atomic::AtomicBool;
-use std::sync::mpsc::{sync_channel, Receiver, SyncSender};
+use std::sync::mpsc::{sync_channel, SyncSender};
 use std::sync::{Arc, Mutex};
 use std::thread;
 use std::thread::JoinHandle;
@@ -40,11 +39,10 @@ use utxo::AnyUtxo;
 pub use period::{PeriodCounter, Periodic};
 
 mod block_extra;
-mod fee;
+mod iter;
 mod period;
 mod pipe;
-mod read_detect;
-mod reorder;
+mod stages;
 mod utxo;
 
 // re-exporting deps
@@ -55,8 +53,12 @@ pub use log;
 pub use structopt;
 
 pub use block_extra::BlockExtra;
+pub use iter::iter;
 pub use pipe::PipeIterator;
 
+#[cfg(feature = "parallel")]
+pub use iter::par_iter;
+
 /// Configuration parameters, most important the bitcoin blocks directory
 #[derive(StructOpt, Debug, Clone)]
 pub struct Config {
@@ -89,6 +91,12 @@ pub struct Config {
     #[structopt(short, long)]
     pub utxo_db: Option<PathBuf>,
 
+    /// Start the blocks iteration at the specified height, note blocks*.dat file are read and
+    /// analyzed anyway to follow the blockchain starting at the genesis and populate utxos,
+    /// however they are not emitted
+    #[structopt(long, default_value = "0")]
+    pub start_at_height: u32,
+
     /// Stop the blocks iteration at the specified height
     #[structopt(long)]
     pub stop_at_height: Option<u32>,
@@ -139,102 +147,6 @@ pub struct FsBlock {
     pub next: Vec<BlockHash>,
 }
 
-struct BlockExtraIterator {
-    handle: Option<JoinHandle<()>>,
-    recv: Receiver<Option<BlockExtra>>,
-}
-impl Iterator for BlockExtraIterator {
-    type Item = BlockExtra;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        match self.recv.recv() {
-            Ok(Some(val)) => Some(val),
-            Ok(None) => {
-                if let Some(handle) = self.handle.take() {
-                    handle.join().unwrap();
-                }
-                None
-            }
-            Err(e) => {
-                error!("error iterating {:?}", e);
-                if let Some(handle) = self.handle.take() {
-                    handle.join().unwrap();
-                }
-                None
-            }
-        }
-    }
-}
-
-/// Return an Iterator of [`BlockExtra`] read from `blocks*.dat` contained in the `config.blocks_dir`
-/// Blocks returned are iterated in order, starting from the genesis to the highest block
-/// (minus `config.max_reorg`) in the directory, unless `config.stop_at_height` is specified.
-pub fn iter(config: Config) -> impl Iterator<Item = BlockExtra> {
-    let (send, recv) = sync_channel(config.channels_size.into());
-
-    let handle = Some(iterate(config, send));
-
-    BlockExtraIterator { handle, recv }
-}
-
-#[cfg(feature = "parallel")]
-/// `par_iter` is used when the task to be performed on the blockchain is more costly
-/// than iterating the blocks. For example verifying the spending conditions in the blockchain.
-/// like [`crate::iter`] accepts configuration parameters via the [`Config`] struct.
-/// A `PREPROC` closure has to be provided, this process a single block and produce a Vec of
-/// user defined struct `DATA`.
-/// A `TASK` closure accepts the `DATA` struct and a shared `STATE` and it is executed in a
-/// concurrent way. The `TASK` closure returns a bool which indicates if the execution should be
-/// terminated.
-/// Note that access to `STATE` in `TASK` should be done carefully otherwise the contention would
-/// reduce the speed of execution.
-///  
-pub fn par_iter<STATE, PREPROC, TASK, DATA>(
-    config: Config,
-    state: Arc<STATE>,
-    pre_processing: PREPROC,
-    task: TASK,
-) where
-    PREPROC: Fn(BlockExtra) -> Vec<DATA> + 'static + Send,
-    TASK: Fn(DATA, Arc<STATE>) -> bool + 'static + Send + Sync,
-    DATA: 'static + Send,
-    STATE: 'static + Send + Sync,
-{
-    use log::debug;
-    use rayon::prelude::*;
-    use std::sync::atomic::Ordering::SeqCst;
-
-    let (send_task, recv_task) = sync_channel(config.channels_size.into());
-    let stop = Arc::new(AtomicBool::new(false));
-
-    let stop_clone = stop.clone();
-    let iter = iter(config);
-
-    let handle = thread::spawn(move || {
-        debug!("start pre-processing thread");
-        for block_extra in iter {
-            if stop_clone.load(SeqCst) {
-                break;
-            }
-            let data_vec = pre_processing(block_extra);
-            for data in data_vec {
-                send_task.send(data).unwrap();
-            }
-        }
-        debug!("ending pre-processing thread");
-    });
-
-    recv_task.into_iter().par_bridge().for_each(|data: DATA| {
-        debug!("iter");
-        let result = task(data, state.clone());
-
-        if result {
-            stop.store(true, SeqCst)
-        }
-    });
-    handle.join().unwrap();
-}
-
 fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinHandle<()> {
     thread::spawn(move || {
         let now = Instant::now();
@@ -242,7 +154,7 @@ fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinHandl
 
         // FsBlock is a small struct (~120b), so 10_000 is not a problem but allows the read_detect to read ahead the next block file
         let (send_block_fs, receive_block_fs) = sync_channel(0);
-        let _read = read_detect::ReadDetect::new(
+        let _read = stages::ReadDetect::new(
             config.blocks_dir.clone(),
             config.network,
             early_stop.clone(),
@@ -257,9 +169,10 @@ fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinHandl
         } else {
             send_ordered_blocks
         };
-        let _reorder = reorder::Reorder::new(
+        let _reorder = stages::Reorder::new(
             config.network,
             config.max_reorg,
+            config.start_at_height,
             config.stop_at_height,
             early_stop,
             receive_block_fs,
@@ -267,7 +180,12 @@ fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinHandl
         );
 
         if !config.skip_prevout {
-            let _fee = fee::Fee::new(receive_ordered_blocks, channel, config.utxo_manager());
+            let _fee = stages::Fee::new(
+                config.start_at_height,
+                receive_ordered_blocks,
+                channel,
+                config.utxo_manager(),
+            );
         }
 
         info!("Total time elapsed: {}s", now.elapsed().as_secs());
@@ -290,18 +208,25 @@ mod inner_test {
     use crate::{iterate, Config};
     use std::sync::mpsc::sync_channel;
 
-    #[test]
-    fn test_blk_testnet() {
-        let conf = Config {
-            blocks_dir: "../blocks".into(),
+    fn test_conf() -> Config {
+        Config {
+            blocks_dir: "blocks".into(),
             network: Network::Testnet,
             skip_prevout: false,
             max_reorg: 10,
             channels_size: 0,
             #[cfg(feature = "db")]
             utxo_db: None,
+            start_at_height: 0,
             stop_at_height: None,
-        };
+        }
+    }
+
+    #[test]
+    fn test_blk_testnet() {
+        let _ = env_logger::try_init();
+
+        let conf = test_conf();
         let (send, recv) = sync_channel(0);
 
         let handle = iterate(conf, send);
@@ -316,16 +241,12 @@ mod inner_test {
     #[cfg(feature = "db")]
     #[test]
     fn test_blk_testnet_db() {
+        let _ = env_logger::try_init();
+
         let tempdir = tempfile::TempDir::new().unwrap();
-        let conf = Config {
-            blocks_dir: "../blocks".into(),
-            network: Network::Testnet,
-            skip_prevout: false,
-            max_reorg: 10,
-            channels_size: 0,
-            stop_at_height: None,
-            utxo_db: Some(tempdir.path().to_path_buf()),
-        };
+        let mut conf = test_conf();
+        conf.utxo_db = Some(tempdir.path().to_path_buf());
+
         let (send, recv) = sync_channel(0);
 
         let handle = iterate(conf.clone(), send);
